@@ -12,7 +12,7 @@
 * Written by Guido Socher <guido@linuxfocus.org> 
 *
 */
-#define VERINFO "version 0.5"
+#define VERINFO "version 0.6"
 #include <syslog.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,7 +36,7 @@ void offandexit(int code)
 	int state = TIOCM_DTR;
 	ioctl(fd, TIOCMSET, &state);
 	usleep(10000);
-	syslog(LOG_NOTICE,"daemon stopped by signal, watchdog off");
+	syslog(LOG_NOTICE,"linuxwd: daemon stopped by signal, watchdog off");
 	if (opt_d) printf("watchdog off, exit\n");
 	closelog();
 	exit(0);
@@ -58,7 +58,7 @@ int setwd( enum serialline whichline, int fd, int onoff, int *wdlinestate){
 	}
 	rval=ioctl(fd, TIOCMSET, wdlinestate);
 	if (rval == -1){
-		syslog(LOG_ERR,"setwd ioctl failure");
+		syslog(LOG_ERR,"linuxwd: setwd ioctl failure");
 	}
 	return(*wdlinestate);
 }
@@ -75,7 +75,7 @@ int kickwd(int fd,int *wdlinestate)
 int runshutdown(int fd,int *wdlinestate)
 {
 	kickwd(fd, wdlinestate);
-	syslog(LOG_NOTICE,"Shutdown by buttonpress");
+	syslog(LOG_NOTICE,"linuxwd: Shutdown by buttonpress");
 	setwd( rts_power, fd, 0, wdlinestate);
 	if (opt_d==0){
 		system("/sbin/shutdown -t2 -h now");
@@ -87,6 +87,25 @@ int runshutdown(int fd,int *wdlinestate)
 	exit(0);
 }
 
+/* write a file to the harddisk in /root and erase it later.
+ * This is to make sure that the disk is still workin ok */
+int writetest(int sleeptime)
+{
+	FILE *fd;
+	char filename[]="/root/.linuxwd.testfile";
+	fd=fopen(filename,"w");
+	if (fd == NULL){
+		fprintf(stderr, "ERROR: can not write file %s\n",filename);
+		syslog(LOG_ERR,"linuxwd: ERROR can not write file %s\n",filename);
+		closelog();
+		sleep(sleeptime); /* let wd expire */
+		exit(1);
+	}
+	fputs("ok\n",fd);
+	fclose(fd);
+	unlink(filename);
+	return(0);
+}
 /* get the current state of the push button 
  * and return it. */
 int getpushbutton(int fd)
@@ -130,6 +149,7 @@ int main(int argc, char **argv)
 	int lstate = 0;
 	int i = 0;
 	int rval;
+	int loopcount;
 	pid_t pid;
 
 	/* The following things are used for getopt: */
@@ -163,6 +183,8 @@ int main(int argc, char **argv)
 
 	/* open device */
 	openlog("linuxwd",LOG_PID,LOG_DAEMON);
+	/* do a write test now so we are sure it will work later */
+	writetest(1);
 	fd = open(argv[optind], O_RDWR | O_NDELAY);
 	if (fd < 0) {
 		fprintf(stderr, "ERROR: can not open \"%s\"\n", argv[optind]);
@@ -178,7 +200,7 @@ int main(int argc, char **argv)
 	rval=ioctl(fd, TIOCMSET, &lstate);
 	if (rval == -1){
 		perror("linuxwd ioctl");
-		syslog(LOG_ERR,"ioctl failed");
+		syslog(LOG_ERR,"linuxwd: ioctl failed");
 		exit(1);
 	}
 	/* switch on the power and kick the wd */
@@ -186,7 +208,7 @@ int main(int argc, char **argv)
 	if (opt_d) printf("watchdog started\n");
 	if (opt_t) printf("watchdog: testmode, will loop very 4 seconds (normally it is every 12 seconds) \n");
 	kickwd(fd, &lstate);
-	syslog(LOG_NOTICE,"watchdog started");
+	syslog(LOG_NOTICE,"linuxwd: watchdog started");
 	if (opt_d==0){
 		/* now we fork to background */
 		if ((pid = fork())<0){
@@ -199,12 +221,14 @@ int main(int argc, char **argv)
 	}
 	/* we are the child and run as a daemon */
 	rval=0;
+	loopcount=0;
 	while(1){
 		if (getpushbutton(fd)){
 			if (opt_d) printf("button press detected\n");
 			runshutdown(fd,&lstate);
 		}
 		rval++;
+		loopcount++;
 		if (opt_t){
 			sleep(4);
 		}else{
@@ -220,6 +244,12 @@ int main(int argc, char **argv)
 		}
 		if (opt_d) printf("kickwd\n");
 		kickwd(fd, &lstate);
+		if (loopcount> 450){
+			/* every hour we test if we can still write a
+			 * file on the harddisk in /root */
+			loopcount=0;
+			writetest(40);
+		}
 	}
 	/* we never get here */
 	return (2);
